package com.landray.sys.modeling.design.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.google.common.base.Strings;
import com.landray.common.core.constant.QueryConstant;
import com.landray.common.core.data.IData;
import com.landray.common.core.dto.IdVO;
import com.landray.common.core.dto.MechanismDTO;
import com.landray.common.core.dto.QueryResult;
import com.landray.common.core.util.PropertyLangUtil;
import com.landray.common.exception.KmssRuntimeException;
import com.landray.common.exception.NoRecordException;
import com.landray.common.util.JsonUtil;
import com.landray.common.util.LangUtil;
import com.landray.common.util.ReflectUtil;
import com.landray.framework.meta.MetaEntity;
import com.landray.framework.meta.MetaProperty;
import com.landray.framework.support.ApplicationContextHolder;
import com.landray.sys.auth.data.PermissionConstant;
import com.landray.sys.auth.util.UserUtil;
import com.landray.sys.lang.annotation.SysLangTransport;
import com.landray.sys.lang.dto.transport.SysLangDataVO;
import com.landray.sys.lang.dto.transport.SysLangTransVO;
import com.landray.sys.lang.spi.ISysLangTransportProvider;
import com.landray.sys.lbpm.dto.template.LbpmFormCategoryVO;
import com.landray.sys.lbpm.support.util.LbpmMechanismUtil;
import com.landray.sys.meta.data.dto.DataQueryRequest;
import com.landray.sys.meta.data.util.SysMetaDataUtil;
import com.landray.sys.modeling.base.api.IModelingTemplateApi;
import com.landray.sys.modeling.base.api.ISysModelingVirtualPolymersApi;
import com.landray.sys.modeling.base.api.SysModelingCommonlyTempApi;
import com.landray.sys.modeling.base.api.auth.ISysVirtualAuthApi;
import com.landray.sys.modeling.base.api.extend.ISysModelingModelExtendInstanceApi;
import com.landray.sys.modeling.base.data.VirtualTemplateFdStatus;
import com.landray.sys.modeling.base.dto.IdVirtualVO;
import com.landray.sys.modeling.base.dto.IdsVirtualDTO;
import com.landray.sys.modeling.base.dto.SysModelingCategoryModelVO;
import com.landray.sys.modeling.base.dto.SysModelingPolymersTreeVO;
import com.landray.sys.modeling.base.dto.SysModelingVirtualPolymersVO;
import com.landray.sys.modeling.base.dto.category.VirtualCategoryTreeRequestVO;
import com.landray.sys.modeling.base.dto.common.IModelingModelVirtualTemplateVO;
import com.landray.sys.modeling.base.entity.SysModelingVirtualPolymersTemplate;
import com.landray.sys.modeling.base.service.AbstractVirtualTemplateService;
import com.landray.sys.modeling.design.metadata.SysModelingVirtualMainProvider;
import com.landray.sys.modeling.design.metadata.SysModelingVirtualPolymersProvider;
import com.landray.sys.xform.constant.DesignStatus;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;
import javax.validation.constraints.NotNull;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RequestMapping({"/api/sys-modeling/categoryPolymers"})
@SysLangTransport(id = "sys-modeling-polymers")
@RestController
@Service
/* loaded from: SysModelingVirtualPolymersService.class */
public class SysModelingVirtualPolymersService extends AbstractVirtualTemplateService<SysModelingVirtualPolymersVO, SysModelingVirtualPolymersTemplate> implements ISysModelingVirtualPolymersApi, IModelingTemplateApi<SysModelingVirtualPolymersVO>, ISysModelingVirtualCategoryService<SysModelingVirtualPolymersVO>, ISysVirtualAuthApi, ISysLangTransportProvider {
    private static final Logger log = LoggerFactory.getLogger(SysModelingVirtualPolymersService.class);
    private final ISysModelingModelExtendInstanceApi modelExtendInstanceService;
    private final SysModelingCommonlyTempApi commonlyTemplateService;

    public /* bridge */ /* synthetic */ void update(VirtualTemplateFdStatus x0) {
        super.update((SysModelingVirtualPolymersVO) x0);
    }

    public SysModelingVirtualPolymersService(ISysModelingModelExtendInstanceApi modelExtendInstanceService, SysModelingCommonlyTempApi commonlyTemplateService) {
        this.modelExtendInstanceService = modelExtendInstanceService;
        this.commonlyTemplateService = commonlyTemplateService;
    }

    public IdVO add(SysModelingVirtualPolymersVO vo) {
        checkCode(vo);
        buildLbpmTemplateFormCategory(vo);
        return super.add(vo);
    }

    private void buildLbpmTemplateFormCategory(SysModelingVirtualPolymersVO vo) {
        SysModelingCategoryModelVO model = (SysModelingCategoryModelVO) this.categoryModelApi.loadByCode(SysMetaDataUtil.metaId(vo.getFdCategoryModelCode())).orElseThrow(NoRecordException::new);
        LbpmFormCategoryVO bpmFormCategory = new LbpmFormCategoryVO();
        bpmFormCategory.setFdModuleCode("sys-modeling");
        bpmFormCategory.setFdSystemCode("INNER_SYSTEM");
        if (Objects.isNull(vo.getFdParent()) || StrUtil.isBlank(vo.getFdParent().getFdId())) {
            bpmFormCategory.setFdName(model.getFdName());
            bpmFormCategory.setFdFormCategoryId(SysMetaDataUtil.wrapId("sys-modeling-polymers", model.getFdCode()));
        } else {
            bpmFormCategory.setFdName(vo.getFdParent().getFdName());
            bpmFormCategory.setFdFormCategoryId(vo.getFdParent().getFdId());
        }
        bpmFormCategory.setFdGroupId(SysMetaDataUtil.wrapId("sys-modeling-polymers", model.getFdCode()));
        bpmFormCategory.setFdModuleTemplEntity(SysMetaDataUtil.wrapId("sys-modeling-polymers", vo.getFdCategoryModelCode()));
        LbpmMechanismUtil.modifyBpmTemplateMechanismsForEach(vo, template -> {
            template.setFdFormCategory(bpmFormCategory);
        });
    }

    private void checkCode(SysModelingVirtualPolymersVO vo) {
        if (Objects.isNull(vo) || StrUtil.isBlank(vo.getFdCode())) {
            throw new KmssRuntimeException("sys-modeling:error.code.must.not.be.null");
        }
        DataQueryRequest request = new DataQueryRequest();
        request.setEntityId(SysMetaDataUtil.wrapId(getEntityName(), vo.getFdCategoryModelCode()));
        request.addCondition("fdCode", vo.getFdCode());
        request.setContent(false);
        if (findAll(request).getTotalSize() > 0) {
            throw new KmssRuntimeException("sys-modeling:error.code.must.be.unique");
        }
    }

    public Optional<SysModelingVirtualPolymersVO> loadById(IdVirtualVO id) {
        Optional<SysModelingVirtualPolymersVO> optional = super.loadById(id);
        if (!optional.isPresent()) {
            return optional;
        }
        SysModelingVirtualPolymersVO polymersVO = (SysModelingVirtualPolymersVO) refineHierarchy(CollUtil.newArrayList(new SysModelingVirtualPolymersVO[]{optional.get()}), SysMetaDataUtil.wrapId(getEntityName(), id.getFdCategoryModelCode())).get(0);
        return Optional.of(polymersVO);
    }

    public QueryResult<SysModelingVirtualPolymersVO> findAllWithChild(VirtualCategoryTreeRequestVO request) {
        if (CollUtil.isEmpty(request.getConditions())) {
            request.addCondition("fdParent", QueryConstant.Operator.isNull, "");
        }
        request.addColumn(new String[]{"fdId", "fdName", "fdHierarchyId", "fdParent", "fdCategoryModelCode"});
        QueryResult<SysModelingVirtualPolymersVO> result = findAll(request);
        if (CollUtil.isEmpty(result.getContent())) {
            return result;
        }
        List<SysModelingVirtualPolymersVO> resultList = new ArrayList<>();
        for (SysModelingVirtualPolymersVO polymersVO : result.getContent()) {
            if (checkReader(polymersVO.getFdId(), polymersVO.getFdCategoryModelCode())) {
                resultList.add(polymersVO);
            }
        }
        buildChildFlag(resultList, request.getEntityId());
        result.setContent(refineHierarchy(resultList, request.getEntityId()));
        return result;
    }

    public List<SysModelingPolymersTreeVO> tree(VirtualCategoryTreeRequestVO request) {
        if (StrUtil.isBlank(request.getFdCategoryModelCode())) {
            log.warn("分类表单code不允许为空");
            return null;
        }
        List<SysModelingPolymersTreeVO> result = new ArrayList<>();
        request.setEntityId(SysMetaDataUtil.wrapId("sys-modeling-polymers", request.getFdCategoryModelCode()));
        if (StrUtil.isNotBlank(request.getFdParentId())) {
            SysModelingVirtualPolymersVO polymers = loadById(IdVirtualVO.of(request.getFdParentId(), request.getFdCategoryModelCode())).orElseThrow(NoRecordException::new);
            result.add(buildPolymersTree(polymers, request));
        } else {
            DataQueryRequest req = new DataQueryRequest();
            req.addCondition("fdParent", QueryConstant.Operator.isNull, (Object) null).addColumn(new String[]{"fdId", "fdName", "fdCode", "fdTreeLevel", "fdHierarchyId"});
            req.setCount(false);
            req.setPageSize(1000);
            req.setEntityId(request.getEntityId());
            List<SysModelingVirtualPolymersVO> voList = findAll(req).getContent();
            for (SysModelingVirtualPolymersVO polymers2 : voList) {
                result.add(buildPolymersTree(polymers2, request));
            }
        }
        return result;
    }

    public void deleteOnCategoryModel(String categoryCode) {
        if (StrUtil.isBlankIfStr(categoryCode)) {
            return;
        }
        DataQueryRequest request = new DataQueryRequest();
        request.setEntityId(SysMetaDataUtil.wrapId(getEntityName(), categoryCode));
        request.setContent(false);
        long totalSize = findAll(request).getTotalSize();
        if (totalSize == 0) {
            if (log.isDebugEnabled()) {
                log.debug("当前分类表单:{}, 无简单分类", categoryCode);
                return;
            }
            return;
        }
        request.setContent(true);
        request.addColumn(new String[]{"fdId"});
        for (int current = 0; current < totalSize; current++) {
            request.setOffset(current);
            List<String> ids = (List) findAll(request).getContent().stream().map((v0) -> {
                return v0.getFdId();
            }).collect(Collectors.toList());
            for (String id : ids) {
                super.delete(IdVirtualVO.of(id, categoryCode));
            }
        }
    }

    public List<SysModelingVirtualPolymersVO> getAllChild(List<IdVirtualVO> fdIds) {
        if (CollUtil.isEmpty(fdIds)) {
            return null;
        }
        Set<String> idSet = new HashSet<>();
        List<SysModelingVirtualPolymersVO> result = new ArrayList<>();
        for (IdVirtualVO vo : fdIds) {
            SysModelingVirtualPolymersVO polymers = loadById(vo).orElseThrow(NoRecordException::new);
            DataQueryRequest req = new DataQueryRequest();
            req.addColumn(new String[]{"fdId", "fdCategoryModelCode", "fdName", "fdCode"});
            req.setPageSize(1000);
            req.setEntityId(SysMetaDataUtil.wrapId(getEntityName(), vo.getFdCategoryModelCode()));
            req.addCondition("fdHierarchyId", QueryConstant.Operator.contains, vo.getFdId());
            req.addCondition("fdTreeLevel", QueryConstant.Operator.gte, polymers.getFdTreeLevel());
            req.addCondition("fdStatus", DesignStatus.Official).addCondition("fdEnable", true);
            List<SysModelingVirtualPolymersVO> content = findAll(req).getContent();
            if (CollUtil.isEmpty(content)) {
                log.warn("无法根据fdId集合查询到对应的简单分类:{}", JsonUtil.toJsonString(fdIds));
                return CollUtil.newArrayList(new SysModelingVirtualPolymersVO[0]);
            }
            for (SysModelingVirtualPolymersVO polymersVO : content) {
                if (!idSet.contains(polymersVO.getFdId())) {
                    result.add(polymersVO);
                    idSet.add(polymersVO.getFdId());
                }
            }
        }
        return result;
    }

    private SysModelingPolymersTreeVO buildPolymersTree(SysModelingVirtualPolymersVO polymers, DataQueryRequest request) {
        DataQueryRequest req = new DataQueryRequest();
        req.setPageSize(request.getPageSize());
        req.setEntityId(request.getEntityId());
        req.setOffset(request.getOffset());
        req.addCondition("fdTreeLevel", QueryConstant.Operator.gte, polymers.getFdTreeLevel()).addCondition("fdHierarchyId", QueryConstant.Operator.contains, polymers.getFdId());
        return convertToTreeAndMarkCommonly(polymers, findAll(req));
    }

    private SysModelingPolymersTreeVO convertToTreeAndMarkCommonly(SysModelingVirtualPolymersVO root, @NotNull QueryResult<SysModelingVirtualPolymersVO> queryResult) {
        Set<String> templateCodeSet = (Set) this.commonlyTemplateService.getByUserAndModelCode(UserUtil.getCurUserInfo().getUserId(), root.getFdCategoryModelCode()).stream().map((v0) -> {
            return v0.getFdTemplateCode();
        }).collect(Collectors.toSet());
        SysModelingPolymersTreeVO result = new SysModelingPolymersTreeVO();
        result.setFdId(root.getFdId());
        result.setFdName(root.getFdName());
        result.setHasMore(((long) (queryResult.getOffset() + queryResult.getPageSize())) < queryResult.getTotalSize());
        if (CollUtil.isEmpty(queryResult.getContent())) {
            return result;
        }
        for (SysModelingVirtualPolymersVO polymers : queryResult.getContent()) {
            result.appendChildren(polymers.getFdId(), polymers.getFdName(), templateCodeSet.contains(polymers.getFdCode()));
        }
        return result;
    }

    public void batchDelete(IdsVirtualDTO dto) {
        if (CollUtil.isEmpty(dto.getFdIds()) || StrUtil.isBlank(dto.getFdCategoryModelCode())) {
            throw new KmssRuntimeException("sys-modeling:error.code.must.not.be.null");
        }
        for (String id : dto.getFdIds()) {
            DataQueryRequest request = new DataQueryRequest();
            request.setEntityId(SysMetaDataUtil.wrapId("sys-modeling-polymers", dto.getFdCategoryModelCode()));
            request.setContent(false);
            request.addCondition("fdParent.fdId", id).addCondition("fdId", QueryConstant.Operator.notIn, dto.getFdIds());
            if (findAll(request).getTotalSize() > 0) {
                throw new KmssRuntimeException("sys-modeling:error.category.delete.has.child.category");
            }
        }
        dto.getFdIds().forEach(id2 -> {
            super.delete(IdVirtualVO.of(id2, dto.getFdCategoryModelCode()));
        });
    }

    public void delete(IdVirtualVO id) {
        if (checkChild(id)) {
            throw new KmssRuntimeException("sys-modeling:error.category.delete.has.child.category");
        }
        super.delete(id);
    }

    private boolean checkChild(IdVirtualVO id) {
        DataQueryRequest request = new DataQueryRequest();
        request.setEntityId(SysMetaDataUtil.wrapId("sys-modeling-polymers", id.getFdCategoryModelCode()));
        request.setContent(false);
        request.addCondition("fdParent.fdId", id.getFdId());
        return findAll(request).getTotalSize() > 0;
    }

    public String getEntityName() {
        return "sys-modeling-polymers";
    }

    protected ISysModelingModelExtendInstanceApi getModelExtendInstanceService() {
        return this.modelExtendInstanceService;
    }

    public boolean checkEditor(String fdId, String categoryCode) {
        DataQueryRequest request = buildAuthRequest(fdId, categoryCode, PermissionConstant.Group.EDITOR);
        return CollUtil.isNotEmpty(findAll(request).getContent());
    }

    public boolean checkReader(String fdId, String categoryModelCode) {
        DataQueryRequest request = buildAuthRequest(fdId, categoryModelCode, PermissionConstant.Group.READER);
        return CollUtil.isNotEmpty(findAll(request).getContent());
    }

    private DataQueryRequest buildAuthRequest(String fdId, String categoryModelCode, PermissionConstant.Group group) {
        DataQueryRequest request = new DataQueryRequest();
        request.addColumn(new String[]{"fdId"});
        request.addCondition("fdId", fdId).addCondition("$$category_auth_check$$", group);
        request.setEntityId(SysMetaDataUtil.wrapId(getEntityName(), categoryModelCode));
        request.setPageSize(1);
        request.setCount(false);
        return request;
    }

    protected boolean needTemplateDataOnMetaEntity() {
        return true;
    }

    public List<SysLangDataVO> exportLang(@RequestBody SysLangTransVO vo) {
        if (StringUtils.isNotEmpty(vo.getExtData())) {
            String entityId = vo.getBaseDoc().getFdEntityId();
            MechanismDTO dto = MechanismDTO.of(SysMetaDataUtil.wrapId("sys-modeling-polymers", vo.getExtData()), entityId);
            vo.setBaseDoc(dto);
        }
        Locale locale = LangUtil.toLocale(vo.getLangCode());
        List<SysLangDataVO> result = new ArrayList<>();
        SysModelingVirtualPolymersVO template = (SysModelingVirtualPolymersVO) super.loadById(IdVirtualVO.of(vo.getBaseDoc().getFdEntityId(), vo.getExtData())).orElseThrow(NoRecordException::new);
        MetaEntity metaEntity = getMetaEntity(template);
        SysLangDataVO nameProperty = new SysLangDataVO();
        nameProperty.setFdEntityName(vo.getBaseDoc().getFdEntityName());
        nameProperty.setFdEntityKey(vo.getExtData());
        nameProperty.setFdEntityId(vo.getBaseDoc().getFdEntityId());
        nameProperty.setLangKey("fdName");
        nameProperty.setOfficialLang(template.getFdName());
        nameProperty.setExportLang(getPropertyByLanguage(metaEntity, template, "fdName", locale));
        result.add(nameProperty);
        SysLangDataVO descProperty = new SysLangDataVO();
        descProperty.setFdEntityName(vo.getBaseDoc().getFdEntityName());
        descProperty.setFdEntityKey(vo.getExtData());
        descProperty.setFdEntityId(vo.getBaseDoc().getFdEntityId());
        descProperty.setLangKey("fdDesc");
        descProperty.setOfficialLang(template.getFdDesc());
        descProperty.setExportLang(getPropertyByLanguage(metaEntity, template, "fdDesc", locale));
        result.add(descProperty);
        return result;
    }

    @Transactional
    public void importLang(@RequestBody SysLangTransVO vo) {
        if (CollectionUtil.isNotEmpty(vo.getLangs())) {
            String entityId = vo.getBaseDoc().getFdEntityId();
            vo.setExtData(((SysLangDataVO) vo.getLangs().get(0)).getFdEntityName());
            MechanismDTO dto = MechanismDTO.of(SysMetaDataUtil.wrapId("sys-modeling-polymers", vo.getExtData()), entityId);
            vo.setBaseDoc(dto);
        }
        Locale locale = LangUtil.toLocale(vo.getLangCode());
        String langCode = LangUtil.getOfficialLang().getLangCode();
        List<SysLangDataVO> sysLangDataVOS = vo.getLangs();
        Map<String, SysLangDataVO> collect = (Map) sysLangDataVOS.stream().filter(sysLangDataVO -> {
            return Strings.isNullOrEmpty(sysLangDataVO.getMechanismKey()) && Strings.isNullOrEmpty(sysLangDataVO.getMechanismId());
        }).collect(Collectors.toMap(item -> {
            return item.getLangKey();
        }, item2 -> {
            return item2;
        }));
        SysModelingVirtualPolymersVO templateVO = (SysModelingVirtualPolymersVO) super.loadById(IdVirtualVO.of(vo.getBaseDoc().getFdEntityId(), vo.getExtData())).orElseThrow(NoRecordException::new);
        String fdNameImport = collect.get("fdName") == null ? null : collect.get("fdName").getImportLang();
        String fdDescImport = collect.get("fdDesc") == null ? null : collect.get("fdDesc").getImportLang();
        if (StringUtils.isNotEmpty(fdNameImport)) {
            templateVO.getDynamicProps().put(PropertyLangUtil.getPropertyNameByLanguage("fdName", locale), fdNameImport);
            if (langCode.equals(vo.getLangCode())) {
                templateVO.setFdName(fdNameImport);
            }
        }
        if (StringUtils.isNotEmpty(fdDescImport)) {
            templateVO.getDynamicProps().put(PropertyLangUtil.getPropertyNameByLanguage("fdDesc", locale), fdDescImport);
            if (langCode.equals(vo.getLangCode())) {
                templateVO.setFdDesc(fdDescImport);
            }
        }
        update((IModelingModelVirtualTemplateVO) templateVO);
    }

    public static String getPropertyByLanguage(MetaEntity metaEntity, IData data, String name, Locale locale) {
        MetaProperty metaProp = metaEntity.getProperty(name);
        if (metaProp == null || !metaProp.isLangSupport()) {
            try {
                return (String) ReflectUtil.getProperty(data, name);
            } catch (ReflectiveOperationException e) {
                throw new RuntimeException(e);
            }
        }
        String langName = PropertyLangUtil.getPropertyNameByLanguage(name, locale);
        Map<String, Object> props = data.getDynamicProps();
        String text = (props == null || langName == null) ? null : (String) props.get(langName);
        if (StringUtils.isBlank(text)) {
            try {
                return (String) ReflectUtil.getProperty(data, name);
            } catch (ReflectiveOperationException e2) {
                throw new RuntimeException(e2);
            }
        }
        return text;
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void afterSaveOrUpdate(SysModelingVirtualPolymersTemplate entity, MetaEntity meta, boolean isAdd) {
        super.afterSaveOrUpdate(entity, meta, isAdd);
        ((SysModelingVirtualPolymersProvider) ApplicationContextHolder.getApplicationContext().getBean(SysModelingVirtualPolymersProvider.class)).removeImmediately(meta.getEntityName(), entity.getFdId());
        ((SysModelingVirtualMainProvider) ApplicationContextHolder.getApplicationContext().getBean(SysModelingVirtualMainProvider.class)).removeImmediately(SysMetaDataUtil.wrapId("sys-modeling-main", entity.getFdCategoryModelCode()), entity.getFdId());
    }
}
